<?php
namespace Vendor\Package;

use FooClass;

namespace Vendor\Package;
use BarClass as Bar;

namespace Vendor\Package;


    use BarClass as Bar;

namespace Vendor
\Package;

namespace Vendor\
Package;
namespace Vendor\

Package;
namespace Vendor\Package;
