phpcs:set Generic.Files.LineLength lineLimit 120
phpcs:set Generic.Files.LineLength absoluteLineLimit 150
<?php

/* This line is fine. This line is fine. This line is fine. This line is fine. This line is fine. */
/* This line is too long. This line is too long. This line is too long. This line is too long. This line is too long. This line is too long. */
/* This line is too long. This line is too long. This line is too long. This line is too long. This line is too long. This line is too long. This line is too long. */
